# -*- coding: utf-8 -*-
from plone.app.linkintegrity import testing
from plone.app.linkintegrity.parser import extractLinks
from plone.app.linkintegrity.tests.base import DXBaseTestCase
from plone.app.linkintegrity.utils import getIncomingLinks
from plone.app.linkintegrity.utils import getOutgoingLinks
from plone.app.linkintegrity.utils import hasIncomingLinks
from plone.app.linkintegrity.utils import hasOutgoingLinks
from plone.app.testing import login
from plone.app.testing import logout
from plone.app.testing import TEST_USER_NAME
from z3c.relationfield import RelationValue
from z3c.relationfield.event import _setRelation
from zc.relation.interfaces import ICatalog
from zope.component import getUtility
from zope.intid.interfaces import IIntIds
from zope.lifecycleevent import modified

import six


class ReferenceGenerationTestCase:

    def test_is_linked(self):
        img1 = self.portal['image1']
        doc1 = self.portal['doc1']
        self._set_text(doc1, '<img src="image1"></img>')
        self.assertTrue(hasIncomingLinks(img1))

    def test_referal_to_private_files(self):
        # This tests the behaviour of the link integrity code when a to
        # be deleted item is referred to by some page the current user
        # has no permission to view. In this case the privacy of the
        # linking user should be protected, so neither the name or url
        # of the linking page should be shown. First we need to create
        # the link in question and set up the permissions accordingly.
        doc = self.portal.doc1
        img = self.portal.image1
        self._set_text(doc, '<a href="image1">Image 1</a>')

        roles = ('Member', )
        self.portal.manage_permission('List folder contents', roles=roles)
        self.portal.manage_permission('Delete objects', roles=roles)
        doc.manage_permission('View', roles=('Manager',), acquire=0)
        doc.manage_permission('Access contents information',
                              roles=('Manager', ), acquire=0)

        logout()
        login(self.portal, 'member')
        checkPermission = self.portal.portal_membership.checkPermission
        self.assertFalse(checkPermission('View', doc))
        self.assertFalse(checkPermission('Access contents information', doc))
        self.assertTrue(checkPermission('View', img))
        self.assertTrue(checkPermission('Access contents information', img))

        # The warning is shown.
        self.assertTrue(hasOutgoingLinks(doc))
        view = img.restrictedTraverse('delete_confirmation')
        results = view()
        self.assertIn('Potential link breakage', results)
        self.assertIn('The item is not accessible.', results)

        # delete linked item and check if the source still has the relation

        # TODO: There is a permission-problem. Deleting the relation
        # When deleting the linked obj the relation is deleted by
        # z3c.relationfield.event.breakRelations. That also fires
        # ObjectModifiedEvent on the linked obj even though the user might not
        # have the permission to edit that obj.
        # Here plone.app.versioningbehavior.subscribers.create_version_on_save
        # for the linked object is triggerted and results in
        # Unauthorized: You are not allowed to access 'save' in this context

        # self.portal.manage_delObjects(img.id)
        self.portal._delObject(img.id, suppress_events=True)

        logout()
        login(self.portal, TEST_USER_NAME)
        modified(doc)
        self.assertFalse(hasOutgoingLinks(doc))
        # doc now has a broken link and no relation :-(

    def test_link_extraction_easy(self):
        doc1 = self.portal.doc1
        self._set_text(doc1, '<a href="doc2">Doc 2</a>')
        self.assertEqual(
            extractLinks(self._get_text(doc1)),
            ('doc2', )
        )

    def test_link_extraction_more_complex(self):
        doc2 = self.portal.doc2
        self._set_text(
            doc2,
            '<a href="doc1">Doc 2</a>' +
            '<a href="folder1/doc3"><img src="image1" /></a>',
        )
        self.assertEqual(
            extractLinks(self._get_text(doc2)),
            ('doc1',
             'folder1/doc3',
             'image1')
        )

    def test_broken_references(self):
        # create a temporary document to test with
        doc1a = testing.create(self.portal, 'Document', id='doc1a')
        doc1 = self.portal.doc1

        self.assertEqual(len(list(getOutgoingLinks(doc1))), 0)
        self._set_text(doc1, '<a href="doc1a">Doc 1a</a>')
        self.assertEqual(len(list(getOutgoingLinks(doc1))), 1)
        self.assertEqual([l.to_object for l in getOutgoingLinks(doc1)],
                         [self.portal.doc1a])

        # Now delete the target item, suppress events and test again,
        # the reference should be gone now.
        self.portal._delObject(doc1a.id, suppress_events=True)
        self.assertEqual([l.to_object for l in getOutgoingLinks(doc1)], [None])

    def test_relative_upwards_link_generates_matching_reference(self):
        doc1 = self.portal.doc1
        doc3 = self.portal.folder1.doc3
        self._set_text(doc3, '<a href="../doc1">go!</a>')
        self.assertEqual(len(list(getOutgoingLinks(doc1))), 0)
        self.assertEqual([l.to_object for l in getOutgoingLinks(doc3)],
                         [doc1])

    def test_unicode_links(self):
        doc1 = self.portal.doc1

        # This tests checks that hasIncomingLinks can now be used safely as it
        # eventually plays well with transaction machinery.
        # Add bad link, should not raise exception and there should not
        # be any references added.
        self._set_text(
            doc1,
            '<a href="ö?foo=bar&baz=bam">bug</a>')

        self.assertEqual([l for l in getOutgoingLinks(doc1)], [])

    def test_reference_orthogonality(self):
        doc = self.portal.doc1
        img = self.portal.image1
        tag = img.restrictedTraverse('@@images').tag()

        # This tests the behavior when other references already exist.
        self.assertEqual([l for l in getOutgoingLinks(doc)], [])
        self.assertEqual([l for l in getIncomingLinks(doc)], [])
        self.assertEqual([l for l in getOutgoingLinks(img)], [])
        self.assertEqual([l for l in getOutgoingLinks(img)], [])

        # Then establish a reference between the document and image as
        # a related item:
        self._set_related_items(doc, [img, ])
        self.assertEqual(self._get_related_items(doc), [img, ])

        # Next edit the document body and insert a link to the image,
        # which should trigger the creation of a link integrity reference:
        self._set_text(doc, tag)

        self.assertEqual([l.to_object for l in getOutgoingLinks(doc)], [img])

        # And the related item reference remains in place:
        self.assertEqual(self._get_related_items(doc), [img, ])

        # Finally, edit the document body again, this time removing the
        # link to the image, which should trigger the removal of the
        # link integrity reference:
        self._set_text(doc, 'where did my link go?')
        self.assertEqual([l.to_object for l in getOutgoingLinks(doc)], [])

        # And again the related item reference remains in place:
        self.assertEqual(self._get_related_items(doc), [img, ])

    def test_delete_confirmation_for_any_reference(self):
        """Test, if delete confirmation shows also a warning if items are
        deleted, which are referenced by other items via a reference field.
        """
        img1 = self.portal['image1']
        doc1 = self.portal['doc1']

        intids_tool = getUtility(IIntIds)
        to_id = intids_tool.getId(img1)
        rel = RelationValue(to_id)
        _setRelation(doc1, 'related_image', rel)

        # Test, if relation is present in the relation catalog
        catalog = getUtility(ICatalog)
        rels = list(catalog.findRelations({'to_id':  to_id}))
        self.assertEqual(len(rels), 1)

        # Test, if delete_confirmation_info shows also other relations than
        # ``isReferencing``.
        info = img1.restrictedTraverse('@@delete_confirmation_info')
        breaches = info.get_breaches()
        self.assertEqual(len(breaches), 1)
        self.assertEqual(len(info.get_breaches()[0]['sources']), 1)


class ReferenceGenerationDXTestCase(
    DXBaseTestCase,
    ReferenceGenerationTestCase,
):
    """Reference generation testcase for dx content types"""

if six.PY2:
    from plone.app.linkintegrity.tests.base import ATBaseTestCase

    class ReferenceGenerationATTestCase(
        ATBaseTestCase,
        ReferenceGenerationTestCase,
    ):
        """Reference generation testcase for at content types"""



# -*- coding: utf-8 -*-
from zope.interface import Interface


class IRetriever(Interface):
    """A retriever for links in a content type.
    """

    def retrieveLinks():
        """ retrieve links """



# -*- coding: utf-8 -*-
from plone.app.linkintegrity import testing
from plone.app.linkintegrity.tests.base import DXBaseTestCase
# from plone.app.linkintegrity.utils import hasIncomingLinks
from plone.app.linkintegrity.utils import getIncomingLinks
from plone.app.linkintegrity.utils import getOutgoingLinks
from plone.app.linkintegrity.utils import hasOutgoingLinks
from plone.app.testing import TEST_USER_NAME
from plone.app.testing import TEST_USER_PASSWORD
from plone.registry.interfaces import IRegistry
from plone.testing.z2 import Browser
from Products.CMFPlone.interfaces import IEditingSchema
from zc.relation.interfaces import ICatalog
from zope.component import getUtility

import six
import transaction
import unittest


class ReferenceTestCase:

    @unittest.skip('Re-enable after https://github.com/plone/plone.app.content/issues/38')  # noqa
    def test_file_reference_linkintegrity_page_is_shown(self):
        doc1 = self.portal.doc1
        file2 = testing.create(self.portal, 'File',
                               id='file2', file=testing.GIF)

        self.assertFalse(hasOutgoingLinks(doc1))
        self._set_text(doc1, '<a href="file2">A File</a>')
        self.assertTrue(hasOutgoingLinks(doc1))
        self.assertIn('file2', self.portal.objectIds())

        token = self._get_token(file2)
        self.request['_authenticator'] = token

        # Make changes visible to test browser
        transaction.commit()

        self.browser.handleErrors = True
        self.browser.addHeader(
            'Authorization',
            'Basic {0:s}:{1:s}'.format(TEST_USER_NAME, TEST_USER_PASSWORD))

        delete_url = '{0:s}/delete_confirmation?_authenticator={1:s}'.format(
            file2.absolute_url(), token)

        # Try to remove but cancel
        self.browser.open(delete_url)

        # Validate text
        self.assertIn('Potential link breakage', self.browser.contents)
        self.assertIn('<a href="http://nohost/plone/doc1">Test Page 1</a>',
                      self.browser.contents)
        self.assertIn('Would you like to delete it anyway?',
                      self.browser.contents)

        # Click cancel button, item should stay in place
        # FIXME! This fails in Archetypes because the redirect
        # plone.app.content.browser.actions.DeleteConfirmationForm.handle_cancel
        # is broken for AT-content.
        self.browser.getControl(name='form.buttons.Cancel').click()
        self.assertEqual(self.browser.url, file2.absolute_url() + '/view')
        self.assertIn('Removal cancelled.', self.browser.contents)
        self.assertIn('file2', self.portal.objectIds())

        # Try to remove and confirm
        self.browser.open(delete_url)
        self.browser.getControl(name='form.buttons.Delete').click()
        self.assertNotIn('file2', self.portal.objectIds())

    def test_unreferenced_removal(self):
        # This tests against #6666 and #7784, simple removal of a not
        # referenced file, which broke zeo-based installations.

        # We simply use a browser to try to delete a content item.
        self.browser.open(self.portal.doc1.absolute_url())
        self.browser.getLink('Delete').click()
        self.assertIn(
            'Do you really want to delete this item?', self.browser.contents)
        self.browser.getControl(name='form.buttons.Delete').click()

        # The resulting page should confirm the removal:
        self.assertIn('Test Page 1 has been deleted', self.browser.contents)
        self.assertNotIn('doc1', self.portal.objectIds())

    def test_renaming_referenced_item(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2

        # This tests makes sure items that are linked to can still be
        # renamed (see the related bug report in #6608).  First we need
        # to create the necessary links:
        self._set_text(doc1, '<a href="doc2">doc2</a>')
        self.assertEqual(
            [i.from_object for i in getIncomingLinks(doc2)], [doc1])

        # Make changes visible to testbrowseropen
        transaction.commit()

        # Then we use a browser to rename the referenced image:
        self.browser.handleErrors = True
        self.browser.open('{0:s}/object_rename?_authenticator={1:s}'.format(
            doc1.absolute_url(), self._get_token(doc1)))

        self.browser.getControl(name='form.widgets.new_id').value = 'nuname'
        self.browser.getControl(name='form.buttons.Rename').click()
        self.assertIn("Renamed 'doc1' to 'nuname'.", self.browser.contents)
        transaction.commit()

        self.assertNotIn('doc1', self.portal.objectIds())
        self.assertIn('nuname', self.portal.objectIds())
        self.assertIn(doc1, [i.from_object for i in getIncomingLinks(doc2)])

        # We simply use a browser to try to delete a content item.
        self.browser.open(doc2.absolute_url())
        self.browser.getLink('Delete').click()
        self.assertIn(
            'Do you really want to delete this item?', self.browser.contents)
        self.assertIn('nuname', self.portal.objectIds())
        # Link breakabe page should be shown
        self.assertIn('Potential link breakage', self.browser.contents)
        self.assertIn('<a href="http://nohost/plone/nuname">Test Page 1</a>',
                      self.browser.contents)
        self.browser.getControl(name='form.buttons.Delete').click()
        self.assertNotIn('doc2', self.portal.objectIds())

    def test_removal_in_subfolder(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2
        folder1 = self.portal.folder1

        # This tests ensuring link integrity when removing an referenced
        # object contained in a folder that is removed.
        self._set_text(doc1, '<a href="folder1/doc4">a document</a>')
        self._set_text(doc2, '<a href="folder1/doc4">a document</a>')

        # Make changes visible to testbrowseropen
        transaction.commit()

        # Then we try to delete the folder holding the referenced
        # document. Before we can do this we need to prevent the test
        # framework from choking on the exception we intentionally
        # throw.
        self.browser.handleErrors = True

        self.browser.open(
            '{0:s}/delete_confirmation?_authenticator={1:s}'.format(
                folder1.absolute_url(), self._get_token(folder1)
            )
        )
        self.assertIn('Potential link breakage', self.browser.contents)
        self.assertIn('<a href="http://nohost/plone/doc1">Test Page 1</a>',
                      self.browser.contents)
        self.assertIn('<a href="http://nohost/plone/doc2">Test Page 2</a>',
                      self.browser.contents)
        self.browser.getControl(name='form.buttons.Delete').click()
        self.assertNotIn('folder1', self.portal.objectIds())

    def test_removal_with_cookie_auth(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2

        # This tests ensures link integrity working correctly without
        # http basic authentication (see the bug report in #6607).
        self._set_text(doc1, '<a href="doc2">doc2</a>')
        transaction.commit()

        browser = Browser(self.layer['app'])
        browser.handleErrors = True
        browser.addHeader('Referer', self.portal.absolute_url())
        browser.open(
            '{0:s}/folder_contents'.format(self.portal.absolute_url()))

        # At this point we shouldn't be able to look at the folder
        # contents (as an anonymous user):
        self.assertIn('login?came_from', browser.url)

        # So we log in via the regular plone login form and additionally check
        # that there is no 'authorization' header set afterwards:
        browser.getControl(name='__ac_name').value = TEST_USER_NAME
        browser.getControl(name='__ac_password').value = TEST_USER_PASSWORD
        browser.getControl('Log in').click()
        self.assertNotIn(
            'authorization', [h.lower() for h in browser.headers.keys()])

        # This should lead us back to the "folder contents" listing,
        # where we try to delete the referenced document.
        browser.open('{0:s}/delete_confirmation?_authenticator={1:s}'.format(
            doc2.absolute_url(), self._get_token(doc2)))
        self.assertIn('Potential link breakage', browser.contents)
        self.assertIn('<a href="http://nohost/plone/doc1">Test Page 1</a>',
                      browser.contents)
        browser.getControl(name='form.buttons.Delete').click()
        self.assertNotIn('doc2', self.portal.objectIds())

    def test_linkintegrity_on_off_switch(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2

        # This tests switching link integrity checking on and off.
        self._set_text(doc1, '<a href="doc2">a document</a>')
        transaction.commit()

        # This should lead us back to the "folder contents" listing,
        # where we try to delete the referenced document. Before we can
        # do this we need to prevent the test framework from choking on
        # the exception we intentionally throw.
        self.browser.handleErrors = True

        self.browser.open(
            '{0:s}/delete_confirmation?_authenticator={1:s}'.format(
                doc2.absolute_url(), self._get_token(doc2)
            )
        )
        self.assertIn('Potential link breakage', self.browser.contents)
        self.assertIn('<a href="http://nohost/plone/doc1">Test Page 1</a>',
                      self.browser.contents)

        # Now we turn the switch for link integrity checking off via the site
        # properties and try again:
        registry = getUtility(IRegistry)
        settings = registry.forInterface(IEditingSchema, prefix='plone')
        settings.enable_link_integrity_checks = False
        transaction.commit()
        self.browser.reload()
        self.assertNotIn('Potential link breakage', self.browser.contents)

    def test_update(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2
        doc4 = self.portal.folder1.doc4

        # This tests updating link integrity information for all site content,
        # i.e. after migrating from a previous version.
        self._set_text(doc1, '<a href="doc2">a document</a>')
        self._set_text(doc2, '<a href="folder1/doc4">a document</a>')

        catalog = getUtility(ICatalog)
        rels = [i for i in catalog.findRelations()]
        for rel in rels:
            catalog.unindex(rel)

        # Just to make sure, we check that there are no references from or to
        # these documents at this point:
        self.assertEqual([i.to_object for i in getOutgoingLinks(doc1)], [])
        self.assertEqual([i.to_object for i in getOutgoingLinks(doc2)], [])

        # An update of link integrity information for all content is triggered
        # by browsing a specific url:
        transaction.commit()
        self.browser.open('{0:s}/updateLinkIntegrityInformation'.format(
            self.portal.absolute_url()))
        self.browser.getControl('Update').click()
        self.assertIn('Link integrity information updated for',
                      self.browser.contents)

        # Now the linking documents should hold the correct link integrity
        # references:
        self.assertEqual(
            [i.to_object for i in getOutgoingLinks(doc1)],
            [doc2, ],
        )
        self.assertEqual(
            [i.to_object for i in getOutgoingLinks(doc2)],
            [doc4, ],
        )

    def test_references_on_cloned_objects(self):
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2

        # This tests ensures that link integrity is correctly setup when
        # cloning an object.
        self._set_text(doc1, '<a href="doc2">a document</a>')

        # Next we clone the document:
        token = self._get_token(doc1)
        self.request['_authenticator'] = token
        doc1.restrictedTraverse('object_copy')()

        self.request['_authenticator'] = token
        self.portal.restrictedTraverse('object_paste')()
        self.assertIn('copy_of_doc1', self.portal)
        transaction.commit()

        # Then we try to delete the document linked by the original document
        # and its clone. Before we can do this we need to prevent the test
        # framework from choking on the exception we intentionally throw.
        self.browser.handleErrors = True

        # Now we can continue and "click" the "delete" action. The confirmation
        # page should list both documents:
        self.browser.open(
            '{0:s}/delete_confirmation?_authenticator={1:s}'.format(
                doc2.absolute_url(), self._get_token(doc2)
            )
        )
        self.assertIn(
            'is referenced by the following items:', self.browser.contents)
        self.assertIn('Potential link breakage', self.browser.contents)
        self.assertIn(
            '<a href="http://nohost/plone/doc1">Test Page 1</a>',
            self.browser.contents
        )
        self.assertIn(
            '<a href="http://nohost/plone/copy_of_doc1"',
            self.browser.contents
        )

    def test_files_with_spaces_removal(self):
        doc1 = self.portal.doc1

        # This tests the behaviour when removing a referenced file that has
        # spaces in its id.  First we need to rename the existing file:
        self.portal.invokeFactory(
            'Document', id='some spaces.doc', title='A spaces doc')

        self.assertIn('some spaces.doc', self.portal.objectIds())
        spaces1 = self.portal['some spaces.doc']

        self._set_text(doc1, '<a href="some spaces.doc">a document</a>')

        # The document should now have a reference to the file:
        self.assertEqual(
            [i.to_object for i in getOutgoingLinks(doc1)], [spaces1])
        transaction.commit()

        # Then we use a browser to try to delete the referenced file.
        # Before we can do this we need to prevent the test framework
        # from choking on the exception we intentionally throw.
        self.browser.handleErrors = True

        self.browser.open(
            '{0:s}/delete_confirmation?_authenticator={1:s}'.format(
                spaces1.absolute_url(), self._get_token(spaces1)
            )
        )
        self.assertIn('Potential link breakage', self.browser.contents)
        self.assertIn(
            '<a href="http://nohost/plone/doc1">Test Page 1</a>',
            self.browser.contents
        )
        self.browser.getControl(name='form.buttons.Delete').click()
        self.assertNotIn('some spaces.doc', self.portal.objectIds())

    def test_removal_via_zmi(self):
        """Delete via ZMI is no longer protedted!"""
        doc1 = self.portal.doc1
        doc2 = self.portal.doc2

        # This tests ensuring link integrity when removing an object via
        # the ZMI.
        self._set_text(doc1, '<a href="doc2">a document</a>')
        self.assertEqual([i.to_object for i in getOutgoingLinks(doc1)], [doc2])

        transaction.commit()
        # Then we use a browser to try to delete the referenced
        # document. Before we can do this we need to prevent the test
        # framework from choking on the exception we intentionally throw.
        self.browser.handleErrors = True

        self.browser.open('http://nohost/plone/manage_main')
        self.browser\
            .getControl(name='ids:list')\
            .getControl(value='doc2').selected = True

        self.browser.getControl('Delete').click()
        self.assertNotIn('doc2', self.portal.objectIds())

    def test_warn_about_content(self):
        folder1 = self.portal.folder1
        self.browser.open(
            '{0:s}/delete_confirmation?_authenticator={1:s}'.format(
                folder1.absolute_url(), self._get_token(folder1)
            )
        )
        self.assertIn('Number of selected', self.browser.contents)
        self.assertIn('2 Objects in all', self.browser.contents)
        self.assertIn('1 Folders', self.browser.contents)
        self.assertIn('0 Published objects', self.browser.contents)


class FunctionalReferenceDXTestCase(DXBaseTestCase, ReferenceTestCase):
    """Functional reference testcase for dx content types"""

    layer = testing.PLONE_APP_LINKINTEGRITY_DX_FUNCTIONAL_TESTING


if six.PY2:
    from plone.app.linkintegrity.tests.base import ATBaseTestCase

    class FunctionalReferenceATTestCase(ATBaseTestCase, ReferenceTestCase):
        """Functional reference testcase for dx content types"""

        layer = testing.PLONE_APP_LINKINTEGRITY_AT_FUNCTIONAL_TESTING



# -*- coding: utf-8 -*-
from plone.app.linkintegrity.tests.base import DXBaseTestCase
from plone.app.linkintegrity.utils import getIncomingLinks
from plone.app.linkintegrity.utils import getOutgoingLinks
from plone.uuid.interfaces import IUUID

import six


class ImageReferenceTestCase:

    def test_image_tag_reference_creation(self):
        doc1 = self.portal.doc1
        img1 = self.portal.image1

        # This tests the correct creation of references used for
        # ensuring link integrity. Any archetype-based content object
        # which refers to other (local) objects by `<img>` or `<a>` tags
        # should create references between those objects on save.
        self._set_text(doc1, img1.restrictedTraverse('@@images').tag())

        self.assertEqual(
            [r.to_object for r in getOutgoingLinks(doc1)],
            [img1, ],
        )
        self.assertEqual([r.to_object for r in getIncomingLinks(doc1)], [])
        self.assertEqual([r.to_object for r in getOutgoingLinks(img1)], [])
        self.assertEqual(
            [r.from_object for r in getIncomingLinks(img1)],
            [doc1],
        )

    def test_image_scale_reference_creation(self):
        doc1 = self.portal.doc1
        img1 = self.portal.image1

        # Linking image scales should also work:
        self._set_text(
            doc1, '<a href="image1/@@images/image_thumb">an image</a>')
        self.assertEqual(
            [r.to_object for r in getOutgoingLinks(doc1)],
            [img1, ],
        )
        self.assertEqual(
            [r.from_object for r in getIncomingLinks(img1)],
            [doc1, ],
        )

    def test_image_resolveuid_reference_creation(self):
        doc1 = self.portal.doc1
        img1 = self.portal.image1

        # Linking via the "resolveuid/UID" method should also work:
        self._set_text(doc1, '<a href="resolveuid/{0:s}">an image</a>'.format(
            IUUID(img1)))
        self.assertEqual(
            [r.to_object for r in getOutgoingLinks(doc1)],
            [img1, ],
        )
        self.assertEqual(
            [r.from_object for r in getIncomingLinks(img1)],
            [doc1, ],
        )


class ImageReferenceDXTestCase(DXBaseTestCase, ImageReferenceTestCase):
    """Image reference testcase for dx content types"""

if six.PY2:
    from plone.app.linkintegrity.tests.base import ATBaseTestCase

    class ImageReferenceATTestCase(ATBaseTestCase, ImageReferenceTestCase):
        """Image reference testcase for dx content types"""
